@@ -27,7 +27,7 @@ class RequestFormatter(logging.Formatter):
     def format(self, record: logging.LogRecord) -> str:
         """Attach the request ID from the request environ to the log record."""
         try:
-            request_id = request.environ.get('request_id', None)
+            request_id = request.environ.get('REQUEST_ID', None)
         except RuntimeError as e:
             request_id = None
         record.requestid = request_id   # type: ignore
@@ -49,6 +49,7 @@ def getLogger(name: str, stream: IO = sys.stderr) -> logging.Logger:
     :class:`logging.Logger`
     """
     logger = logging.getLogger(name)
+    # logger.propagate = False
     config = get_application_config()
 
     # Set the log level from the Flask app configuration.
